from ._assert import assert_likelihood


def normalize_likelihood(lik):
    if not isinstance(lik, (tuple, list)):
        lik = (lik,)

    lik_name = lik[0].lower()
    lik = (lik_name,) + lik[1:]
    assert_likelihood(lik_name)
    return lik
